%% FSP Algorithm for 2-species (non time varying).
clear all
%% Define Stoichiometry and Propensity Functions
example = '4_state_gene_expression';

switch example
    case 'mrna_protein'
        % Stoichiometry matrix
        S = [1 -1  0  0;...
            0  0  1 -1];
        
        % Parameters
        k1 = 30;
        k2 = 30;
        g1 = 0.2;
        g2 = 0.5;
        
        % Propensity Functions
        w = @(x,y)[k1*ones(size(x)), g1*x, k2*x, g2*y];
        
        % Simulation time and initial condition
        t_Array = linspace(0,1,3);
        x0 = [0,0];
        
        % FSP Shape functions
        f = @(x,y)[x,y];     % Truncation shape function -- the first four MUST be [-x -y x y]
    case '4_state_gene_expression'
        
        % Stoichiometry matrix
        S = [1 -1  0  0;...
            0  0  1 -1];
        
        % Parameters
        k12 = 0.1;
        k23 = 0.5;
        k34 = 0.1;
        k21 = 0.1;
        k32 = 0.1;
        k43 = 0.02;
        kr1 = 0;
        kr2 = 10;
        kr3 = 20;
        kr4 = 100;
        gamma = 1;
        
        % Propensity Functions
        w = @(x,y)[k12*(x==1)+k23*(x==2)+k34*(x==3),...
            k21*(x==2)+k32*(x==3)+k43*(x==4),...
            kr1*(x==1)+kr2*(x==2)+kr3*(x==3)+kr4*(x==4),...
            gamma*y];
        
        % Simulation time and initial condition
        t_Array = linspace(0,100,10);
        x0 = [1,0];  % Initial condition is (S1,0).
        
        f = @(x,y)[x,y];     % Truncation shape function -- the first four MUST be [-x -y x y]
        % FSP Shape functions
    case 'toggle_switch'
        
        S = [1 -1  0  0;...
            0  0  1 -1];
        
        k01 = 1;
        k11 = 50;
        M1 = 15;
        eta1 = 5;
        gamma1=1;
        
        k02 = 1;
        k12 = 50;
        M2 = 15;
        eta2 = 5;
        gamma2=1;
        
        w = @(x,y)[k01 + k11*M1^eta1./(M1^eta1 + y.^eta1),...
            gamma1*x,...
            k02 + k12*M2^eta2./(M2^eta2 + x.^eta2),...
            gamma2*y];
        
        t_Array = linspace(0,100,10);
        f = @(x,y)[x,y,(x-4).^2.*(y-4).^2];     % Truncation shape function -- the first four MUST be [-x -y x y]
        x0 = [1,0];  % Initial condition is (S1,0).
        
end
%% Define Boundary shape functions
b = max(1,f(x0(1),x0(2)));
eps_max = 0.01;
tic
[Z,b,Z_Array] = Run_2_Dimension_FSP(S,w,t_Array,f,b,eps_max,x0);
firstrun=toc
tic
[Z,b,Z_Array] = Run_2_Dimension_FSP(S,w,t_Array,f,b,eps_max,x0);
secondrun=toc


%% Plot results
figure(1); clf;
subplot(2,2,1);
contourf([0:size(Z,2)-1],[0:size(Z,1)-1],log10(Z))
c = colorbar;
c.Label.String = 'Log_{10} P(x,y)';
ylabel('Species 1');xlabel('Species 2');
set(gca,'fontsize',14)

subplot(2,2,3)
stairs([sum(Z),sum(Z(:,end))],'linewidth',2);
xlabel('Species 2');ylabel('Prob. Mass');
set(gca,'fontsize',14,'xlim',[0,size(Z,2)])

subplot(2,2,2)
stairs([0:size(Z,1)]',[sum(Z,2);sum(Z(end,:))],'linewidth',2);
xlabel('Species 1');ylabel('Prob. Mass');
set(gca,'fontsize',14,'xlim',[0,size(Z,1)])
