function [Z,b,Z_Array] = Run_2_Dimension_FSP(S,w,t_Array,f,b,eps_max,x0)
%% Initialize states.
M = size(S,2);   % Number of reactions.
N_bd = length(b); % Number of boundary functions

err=1;  % Initialize the error in the FSP.

if nargin<7
    x0=[0,0];
end

while err > eps_max
    %% Compute Inf. Gen. Matrix.
    Nmax = ceil(b(1:2));                    % Find the maximum number of each species.
    

    Nst = prod(Nmax+1);                     % Total number of states.
    Y = zeros(Nst,1);X = zeros(Nst,1);      
    Y(:) = repmat([0:Nmax(2)],Nmax(1)+1,1); % y-value of all states
    X(:) = repmat([0:Nmax(1)]',1,Nmax(2)+1);% x-value of all states
    
    inds = [1:Nst];                         % indices of the FSP states
    props = w(X,Y);                         % propensities of all reactions at all states
    A_Square = spalloc(Nst,Nst,Nst*(M+1));  % allocate Inf.Gen.Mat.
    C = zeros(N_bd,Nst);                    % allocate output matrix
    for mu = 1:M
        stoich_1D = S(1,mu)+S(2,mu)*(Nmax(1)+1); % transform mu^th stoichiometry vector into 1D coordinates
        indsNew = inds + stoich_1D;              % find state after mu^th reaction       
        XNew = X+S(1,mu); YNew = Y+S(2,mu);      % find state after mu^th reaction
        fNew = f(XNew,YNew);                     % compute boundary functions at new state.   
        
        A_Square = A_Square-spdiags([props(:,mu)],0,Nst,Nst);
                                                 % Add diagonal to the
                                                 % reduced inf.gen.mat.
        
        Valid_constraints = (fNew<=b);      % determine which reactions STAY in X_J
        props_keep = props(:,mu);           
        props_keep(min(Valid_constraints,[],2)~=1) = 0;
        A_Square = A_Square+spdiags(props_keep,-stoich_1D,Nst,Nst);
                                            % Enter the props that stay in
                                            % X_J into the inf.gen.mat.
        
        for j=1:N_bd                        %specify the output matrix.
            props_reject = props(:,mu)./abs(sum(Valid_constraints-1,2));
            props_reject(Valid_constraints(:,j)==1) = 0;
            C(j,:) = C(j,:)+(props_reject)';
        end
    end
    
    Valid_constraints = min((f(X,Y)<=b),[],2);
    
    % Define initial probability distribution
    ind_0 = x0(2)*(Nmax(1)+1)+x0(1)+1;
    P0_full = sparse(zeros(size(A_Square,1),1));
    P0_full(ind_0) = 1; % Specify initial condition.
    P0 =  [P0_full(Valid_constraints,1);spalloc(N_bd,1,0)];
    
    % Recuse the A matrix to the important
    A_Red = A_Square(Valid_constraints,Valid_constraints);  % Find A_J
    B_Red = C(:,Valid_constraints);                         % Find A_{Ji,J}
    
    A_fill = [[A_Red;B_Red],spalloc(size(A_Red,1)+N_bd,N_bd,0)]; % Combine to find FSP Inf.Gen.Mat.
    
    
    [P, ~, ~, ~, P_array] = mexpv_modified(max(t_Array), A_fill, P0, 1e-6, 30, 2, t_Array,eps_max,[length(A_fill)-N_bd+1:length(A_fill)]);
    % Use Krylov method to integrate the ODE dP/dt = A*P, assuming that A
    % is an inf.gen.mat.
    
    sinks = P(end-N_bd+1:end);   % Find the probability in the sinks.
    err = sum(sinks);            % Sum the prob. in the sinks.
    if err>eps_max               % If necessary, expand the projection space.
        b(sinks>eps_max/10) = b(sinks>eps_max/10)*1.2;
    end
    
end
Valid_constraints = min((f(X,Y)<=b),[],2);

Z = zeros(size(X));
Z_Array = zeros([size(P_array,2),Nmax+1]);
for i = 2:size(P_array,2)
    Z(Valid_constraints) = P(1:end-N_bd);
    Z = reshape(Z,Nmax+1);
    Z_Array(i,:,:) =Z;
end



